function ELextract(cfg)
%TobiiExtract
%
%   Opens a converted Eyelink-generated Data File (.asc) and processes
%   the data ready for analysis.
%
%   NB: The filenames for the behavioural data should be in the same
%   alphabetical order as the filenames for the eye data. The behavioural
%   and eye data files selected for merging are printed to the calibration
%   window for checking
%
%   If you use this script in a publication, please cite [APA 6th ed.]:
%   
%   Street, C. N. H. (2014). ELextract [MATLAB script]. Retrieved by
%   request from cstreet1986@gmail.com
%   
%
%   This script makes use of the fixationRecalibration function of Vadillo
%   et al., 2014. If you use this script in a publication, you should cite
%   their paper:
%
%   "The eye tracking data were extracted and recalibrated using Vadillo,
%   Street, Beesley & Shanks (2014) fixation recalibration function".
%
%    Vadillo, M. A., Street, C. N. H., Beesley, T., & Shanks, D. R.
%      (in press/2015). A simple algorithm for the offline recalibration
%      of eye-tracking data through best-fitting linear transformation.
%      Behavior Research Methods.
%
%
%       
%                       ***************************************************
%                       Created by Chris Street (cstreet1986@gmail.com),
%                       Dec 2014. Credit to Robert Brookover for putting
%                       me on to the feof and strread functions.
%                       ***************************************************
%
%


%% set defaults

% addpath(genpath(cd));

if nargin == 0
    cfg=initialise;
% cfg=configCheck(cfg); %check everything is filled
end

 
%% get filename(s)

[fls_eye eyepathname fls_bhr] = getFilename(cfg);

%% parse data & analyse

if ischar(fls_eye)
    nF = 1;
else
    nF = length(fls_eye);
end

if cfg.readmat == 1 && (length(fls_eye)~=length(fls_bhr) && nF ~= 1)
    error('No of ASC files must equal no of MAT files');
end

for i = 1:nF
    
    %make sure filename is in correct format
    cfg.flname = fnameFormat(fls_eye, i);
    cfg.bhrname = fnameFormat(fls_bhr, i);

    fprintf('\n*** %s + %s ***\n\n', cfg.flname, cfg.bhrname);
    

%% READ EYE DATA FILE
    
    if exist('datafiles/Processed Mat','dir')
        addpath(strcat(cd, '/datafiles/Processed Mat'));
    end
    %check to see if it already exists
    processCheck = cfg.flname(1:length(cfg.flname)-4);
    processCheck = strcat(processCheck, '_processed.mat');
    if exist(processCheck, 'file') && ~exist('getDat', 'var')
        getDat = input('Process file already exists. Reprocess? 1=Yes [default]: ');
    elseif ~exist('getDat','var')
        getDat = 1;
    end
    
    if isempty(getDat)
        getDat = 1;
    end
    
    if cfg.readmat == 1
        load(cfg.bhrname);
        data = eval(cfg.dataVarName);
    end
    
    
    % ** run through txt file
    if getDat == 1
        if cfg.readmat == 1
            [eyeData e] = file_fUnknown(cfg.flname, cfg, data);
        else
            [eyeData e] = file_fUnknown(cfg.flname, cfg);
        end
    elseif exist(processCheck, 'file')
        load(processCheck); %load data
    else
        if cfg.readmat == 1
            [eyeData e] = file_fUnknown(cfg.flname, cfg, data);
        else
            [eyeData e] = file_fUnknown(cfg.flname, cfg);
        end
    end
    
    %save data thus far
    saveData(cfg.flname, eyeData, cfg, e);
   
%% recalibrate the data if requested

    if cfg.adjustROI == 2
        
        [eyeData fixlong] = recalibrate(e.fixes, cfg, eyeData);
        saveData(cfg.flname, eyeData, cfg, e, fixlong);
    end
    
   
%% WRITE DATA TO FILE

    if cfg.writeraw == 1
        
        writerawELfile(e.fixes, cfg.flname, cfg);
        
    end
    
    
    %------------------------
    %WRITE PROCESSED DATA TO FILE
    
    %NB: These sections will not work. The reason is that each experiment has
    %its own unique measures that you will want to collect. Not being able to
    %know those in advance, I was not able to appropriately adjust this. You
    %have two options. Either adapt the "writerawELfile" and
    %"writeProcessedFile" scripts, and then set the relevant cfg fields to
    %allow writing, or simply leave the relevant cfg fields set to zero so that
    %the writing to csv files is skipped. All your data will save as .mat files
    %in the current directory
    
    if cfg.writesumfile == 1
        if exist(cfg.dataVarName, 'var')
            DATA = eval(cfg.dataVarName);
            writeProcessedFile(eyeData, cfg.flname, cfg, cfg.bhrname, DATA);
        else
%             error('need to use the behavioural data');
            writeProcessedFile(eyeData, cfg.flname, cfg, cfg.bhrname);
        end
    end

    close('all');
    
    %% move data
    
    if cfg.movefile == 1
        %move .mat processed file
        if ~exist('datafiles/Processed Mat', 'dir')
            mkdir('datafiles/Processed Mat');
        end

        dest = strcat(cd, '/datafiles/Processed Mat');
        movefile(strcat(cfg.flname(1:end-4),'_processed.mat'), dest, 'f');
    end
     
end


disp('Finished');
fprintf('\n\nCitation:\n\nStreet, C. N. H. (2014). ELextract [MATLAB script].\nRetrieved by request from cstreet1986@gmail.com\n\n');

end



%% **************************************************
%****************************************************
%                EMBEDDED FUNCTIONS
%****************************************************
%****************************************************



%% *****************

    function [fls_eye eyepathname fls_bhr bhrpathname] = getFilename(cfg)
        
        %if we're reading all files
        if cfg.readall == 0
            fls_eye.name = strcat(filename, '.asc'); %create name for single file

        else

%             fprintf('\n.TXT and .MAT files must be in same directory\n');
            %select .asc files
            [fls_eye eyepathname] = selectFiles('asc',1);
            addpath(eyepathname);
            
        end
        
        
        %if we're reading all files
        if cfg.readmat == 0
            fls_bhr.name = strcat(filename, '.mat'); %create name for single file

        elseif cfg.readmat == 1

%             fprintf('\n.TXT and .MAT files must be in same directory\n');
            %select .asc files
            
            [fls_bhr bhrpathname] = selectFiles('mat',1);
            addpath(bhrpathname);

        else
            
            fls_bhr = [];
            bhrpathname=[];
        end

    end

    %% ---
    
    
    function flname = fnameFormat(fls_asc, i)
    
        %make sure variable is of correct type
        if isstruct(fls_asc)
            flname = fls_asc(i).name;
        elseif iscell(fls_asc)
            flname = fls_asc{i};
        elseif ischar(fls_asc)
            flname = fls_asc;
        end
        
    end