function [eyeData e] = file_fUnknown(filename, cfg, data)

% Read data with unknown format and parse eye movement data. Saves data to
% the eyeData(i) structure, with each (i)th eyeData structure corresponding
% to a trial in the order in which they were presented. The fields within
% eyeData store each fixation within the specified section of the trial:
% either pre-decision, post-decision or during the presentation of an
% asterisk
%
% Credit to robert.p.brookover@gmail.com for the reading of a file with
% unknown format: 
%    http://www.mathworks.co.uk/matlabcentral/newsreader/view_thread/292908


if nargin < 3
    data=[];
end

[e eyeData] = prepVariables(cfg);

fid = openEyeData(filename);

                                           
%read each new line of the file
while ~feof(fid)
    e.idx = e.idx + 1;
    
    %display every ith index
    if rem(e.idx,cfg.idxDisp) == 0 && cfg.idxDisp>0
        fprintf('idx: %i\n',e.idx);
    end
    
    dataLine = fgetl(fid);
    textParse = strread(dataLine, '%s');
    
    if e.idx == cfg.debugIdx
        disp('To debug, breakpoint at line 37 of file_fUnknown');
    end
    
    %need textParse to have at least 3 cells. If it doesn't, we are not in
    %the relevant row. Ignore all other commands. This will also prevent
    %the code from erroring when it tries to read textParse that has too
    %few elements.
    if size(textParse, 1) >= min([min(cfg.ELidentifiers.recordpos) 3])
        
    
%--------- ** GET TRACKER ERROR RATE**
        if size(textParse, 1) >= 4
            if length(textParse) == 18 && strcmp(textParse(4), 'VALIDATION')
                [eyeData, cfg, e] = getTrackerError(eyeData, data, textParse, 0, e, cfg);
            elseif length(textParse) == 18
                [eyeData, cfg, e] = getTrackerError(eyeData, data, textParse, 1, e, cfg);
            end
        end

        
        
%--------- ** TRIAL ONSET (RECORD FIELD) **
        if size(textParse, 1) >= cfg.ELidentifiers.recordpos(1)
             if strcmp(textParse(cfg.ELidentifiers.recordpos(1)), cfg.ELidentifiers.recordname{1}) ...
        %                     && trialno ~= 0

                 [e eyeData] = setTrialOnset(e, eyeData, cfg, textParse);
             end
        end
               
%--------- ** TRIAL END (RECORD FIELD) **
        if size(textParse, 1) >= max([cfg.ELidentifiers.recordpos(end) cfg.ELidentifiers.recordposval(end)])
             if strcmp(textParse(cfg.ELidentifiers.recordpos(end)), cfg.ELidentifiers.recordname{end}) ...
                        && e.trialno ~= 0

                    e.trialend = 1;
                    eyeData(e.trialno).section(e.secno).secEnd = ...
                        str2double(textParse(cfg.ELidentifiers.recordposval(end)));
                    
                    if isempty(eyeData(e.trialno).section)
                        fprintf('\nidx: %i\n',e.idx);
                        disp('possible error: eyeData.section missing');
                    end

             end
        end
        
        
%--------- ** START OF EACH SECTION OF THE TRIAL (SECTION FIELD) **
        
        [e eyeData] = checkSectionStart(e, eyeData, cfg, textParse);
         
        %currently saccading?
        [e eyeData] = checkSaccade(e, eyeData, textParse);
         
         
%--------- ** FIXATION START **
        [e eyeData] = checkFixationStart(e, eyeData, textParse);
        
        
%--------- ** FIXATION END **
        
        %ASSUME THE FOLLOWING STRUCTURE IN DATAFILE:
        %   EFIX <eye used> <onset> <offset> <dur> <aveX> <aveY>
        %   <avePupilSize>
        %
        %most of the calculations happen at this point.
        
        %what section of the trial are we in, if any?
        if strcmp(textParse(1),'EFIX')
            e.fixSectionEnd = e.secno;            
            e.fixate = 0; %no longer fixating
            
            [e eyeData] = endFixation(e, eyeData, cfg, textParse);
        end

%--------- ** MISC TRIAL INFORMATION **

        [e eyeData] = CheckTrialInfo(e, eyeData, cfg, textParse);
        
        
    end %if textParse long enough to start checking for fixations etc
    
end %while feof ~= 1

%close file
fclose(fid);
end

    
%---------------------------
% MISC EMBEDDED FUNCTIONS
%---------------------------


    function [e eyeData] = prepVariables(cfg)
        
        %predefine variables
        e.idx=0;
        e.usefirstfix = cfg.firstfix;
        e.firstfixofSection = 0;
        e.sfix_InOutTrial = 0; %0=start fixation before trial, 1=during trial

        %predefine structure
        eyeData=struct('section',[]);

        e.xcountr=1;
        e.ycountr=1;
        
        e.trialno=0;
        e.secno=0;
        e.fixationSplit = 0;
        e.fixcount = 0;

        e.trialend = 1; %trial hasn't started yet

        e.fixate=0;
        e.mindur = cfg.minFixDur;
        
        e.infoBefore = cfg.ELidentifiers.trialInfoBefore;
        
        e.adjustedROI = 0;
        
        e.fixes = [];
        
    end
    

%--------------------
    function fid = openEyeData(filename)
        
        %open file
        fid = fopen(filename);
        if fid == -1
            addpath(genpath(cd)); %check subfolders
            addpath(ascpath);
            fid = fopen(strcat(ascpath, filename));
            if fid == -1
                error(['Error Opening ',filename]);
            end
        end
    end


%--------------------
    function [eyeData cfg e] = getTrackerError(eyeData, data, textParse, checkVal, ...
                                            e, cfg)

        if ~isfield(eyeData(1), 'section') || isempty(eyeData(1).section)
            eyeData = data;
        end
    
        %grab the error
        if checkVal == 0
            eyeData(e.trialno+1).rightErrAve = str2double(textParse(10));
            eyeData(e.trialno+1).rightErrMax = str2double(textParse(12));

       elseif checkVal == 1
            eyeData(e.trialno+1).rightErrAve = str2double(textParse(10));
            eyeData(e.trialno+1).rightErrMax = str2double(textParse(12));
        end
        
        %adjust ROIs if necessary
        if cfg.adjustROI == 1 && e.adjustedROI == 0
            
            
            %convert calibration error from degrees to pixels
            errW = str2double(textParse(12)) * cfg.pixperdegW;
            errH = str2double(textParse(12)) * cfg.pixperdegH;
            
            %adjust ROIs
            for i = 1:length(cfg.ROI.x1)
                cfg.ROI.x1(i) = cfg.ROI.x1(i) - errW;
                cfg.ROI.x2(i) = cfg.ROI.x2(i) + errW;
                cfg.ROI.y1(i) = cfg.ROI.y1(i) - errH;
                cfg.ROI.y2(i) = cfg.ROI.y2(i) + errH;
            
            end
            e.adjustedROI = 1;
            
        end
        
    end
    

%--------------------
    function [e eyeData] = setTrialOnset(e, eyeData, cfg, textParse)
       
        if ~isfield(e, 'trialno')
            e.trialno = 1;
        else
            e.trialno = e.trialno+1;
        end
        
        %check to see if we have a duplicate trial
        if cfg.trialDuplicates == 1
            if ~isfield(e,'curtrial')
                e.lasttrial = -1;
                e.curtrial = str2double(textParse(cfg.ELidentifiers.recordposval(1)));
            else
                e.lasttrial = e.curtrial;
                e.curtrial = str2double(textParse(cfg.ELidentifiers.recordposval(1)));
            end
            
            %if there's a match, rewrite
            if e.lasttrial == e.curtrial
                e.trialno = e.trialno-1; %rewrite over last trial
%                 eyeData(e.trialno) = []; %delete the trial
                e.fixate = 0; %ignore any fixation overlaps
                e.trialdupe = 1; %identify the last trial as a dupe
                e.splitFixation = 0;
            else
                e.trialdupe = 0;
            end
        
         end
        
        
         e.fixcount = 0; %reset count of fixations
         e.trialend = 0; %start collecting fixations
         e.starttime = str2double(textParse(2));
         e.trialOnset = e.starttime; %take a copy
         e.secno = 1;
         eyeData(e.trialno).section(e.secno).secStart = e.starttime;
             

         if e.trialno == cfg.debugTrial
             fprintf('\nstop at around line 235 for data gathering\n');
         end


         if e.fixate == 1
             e.splitFixation = 1;
         end

        %display trial number
        if rem(e.trialno, cfg.trialDisp) == 0,
            fprintf('TRIAL %i\n', e.trialno);
        end
        
        if ~isfield(e,'trialdupe')
            e.trialdupe = 0;
        end
    end


%--------------------
    function [e eyeData] = checkSectionStart(e, eyeData, cfg, textParse)
        
        for j = 1:length(cfg.ELidentifiers.secpos) %for the start/end msgs
            if length(textParse) >= cfg.ELidentifiers.secpos(j)

                compare1 = textParse(cfg.ELidentifiers.secpos(j));
                compare2 = cfg.ELidentifiers.secname{j};

                if strcmp(compare1, compare2)

                    %identify the start of this section of this trial
                    e.secno = j+1;

                    %if currently fixating somewhere and it hasn't come to
                    %and end by the start of this section
                    if e.fixate == 1

                        %draw a line at the boundary of the sections, and
                        %say that the fixation ended at the boundary for
                        %the last sectoin, and that it starts at the
                        %boundary for the current section. This will get
                        %calculated under the EndFixation section
                        e.splitFixation = 1;
                        

                    end

                    %onset and offest time of this section
                    if e.trialno>0
                        eyeData(e.trialno).section(e.secno).secStart = str2double(textParse(2)); %get section onset time
                        if ~(e.trialno==1 && e.secno==1)
                            if e.secno == 1
                                eyeData(e.trialno-1).section(length(cfg.ELidentifiers.secpos)).secEnd = ...
                                    eyeData(e.trialno).section(e.secno).secStart;
                            else
                                eyeData(e.trialno).section(e.secno-1).secEnd = ...
                                    eyeData(e.trialno).section(e.secno).secStart;
                            end
                        end
                    end
            
                    break;

                end
            end
        end
        
    end
    

%--------------------
    function [e eyeData] = checkSaccade(e, eyeData, textParse)
        
       %** SACCADE START/END **
        if strcmp(textParse(1), 'SSACC')
            e.saccade = 1;
        elseif strcmp(textParse(1), 'ESACC')
            e.saccade = 0;
        end

    end
    
    
%--------------------
    function [e eyeData] = checkFixationStart(e, eyeData, textParse)
        if strcmp(textParse(1),'SFIX')

            e.starttime = str2double(textParse(3)); %get fixation onset time
            e.fixate = 1;

            if ~isfield(e, 'splitFixation')
                e.splitFixation = 0;
            end

            if e.splitFixation == 0
                e.fixSectionStart = e.secno;
                e.fixSectionStartTime = e.starttime;
            end


        end
    end
    
%--------------------

    function [e eyeData] = endFixation(e, eyeData, cfg, textParse)
    
    %--------->> -- (IN SAME SECTION AS START OF FIXATION) --
    if e.splitFixation == 0
        
        if e.trialno > 0

            for j = 1:length(cfg.firstfixsec)
                if cfg.firstfixsec(j) == e.secno
                    skipsec=1;
                    break;
                else
                    skipsec=0;
                end
            end
            
            if ~exist('skipsec','var')
                skipsec = 0;
            end
            
            %ignoring this fixation?
            if e.usefirstfix == 0 && e.fixcount == 0 && skipsec == 1

                %add one to the fixation counter, and ignore fixation
                e.fixcount = e.fixcount + 1;

            %collect eye data
            else

                if e.fixcount == 0
                    e.fixcount = e.fixcount+1; %add one to fixation counter
                end

                e.newdur = str2double(textParse(5));

                if e.newdur >= e.mindur
                    %left or right eye
                    eyeData(e.trialno).section(e.secno).eyeused = textParse(2);
                    eyeData(e.trialno).section(e.secno).x(e.fixcount) = str2double(textParse(6));
                    eyeData(e.trialno).section(e.secno).y(e.fixcount) = str2double(textParse(7));
                    eyeData(e.trialno).section(e.secno).tstamp(e.fixcount) = str2double(textParse(3)) - e.trialOnset;
                    eyeData(e.trialno).section(e.secno).dur(e.fixcount) = e.newdur;

                    %dont bother checking the ROIs if we're going to
                    %process them later
                    if cfg.adjustROI ~= 2
                        eyeData = checkROI(eyeData, e, cfg, str2double(textParse(6)), ...
                                            str2double(textParse(7)), e.trialno, e.secno, e.fixcount);
                    end
                                        
                    if isempty(e.fixes)
                        e.fixes = [e.trialno e.secno e.fixcount str2double(textParse(6)) ...
                            str2double(textParse(7)) eyeData(e.trialno).section(e.secno).tstamp(e.fixcount)...
                            e.newdur];
                    else
                        e.fixes = [e.fixes; e.trialno e.secno e.fixcount str2double(textParse(6)) ...
                            str2double(textParse(7)) eyeData(e.trialno).section(e.secno).tstamp(e.fixcount)...
                            e.newdur];
                    end
                    
                    e.fixcount = e.fixcount+1;

                end

            end %if first fix / not first fix
            
            clear skipsec;
        end


%--------->> -- (IN DIFFERENT SECTION FROM THE START OF FIXATION) -- 
    elseif e.splitFixation == 1
        
        if e.trialno > 0

            %overlap from section 1 to 2, etc
            if e.fixSectionEnd > e.fixSectionStart
                e.nSectionOverlap = e.fixSectionEnd - e.fixSectionStart;
                e.secIDs = [e.fixSectionStart:e.fixSectionEnd];
                e.trialsplit = 0;

            %overlap from section 6 to section 1, etc (looping around
            %to start of trial)
            elseif e.fixSectionStart > e.fixSectionEnd

                e.nSectionOverlap = (length(cfg.ELidentifiers.secpos)+1-...
                        e.fixSectionStart)+e.fixSectionEnd;
                e.secIDs = [e.fixSectionStart:length(cfg.ELidentifiers.secpos)+1 ...
                          1:e.fixSectionEnd];
                e.trialsplit = 1;

            end



            %get fixation duration for this section
            if e.trialsplit == 0  %more than one section overlapped

                e.fixdur(1) = eyeData(e.trialno).section(e.fixSectionStart).secEnd...
                                            - e.fixSectionStartTime;
                e.fixon(1) = e.fixSectionStartTime - e.trialOnset;
                
                ctr=2;
                if length(e.secIDs) > 2
                    for i = 2:length(e.secIDs)-1
                        e.fixdur(ctr) = eyeData(e.trialno).section(e.secIDs(i)).secEnd - ...
                            eyeData(e.trialno).section(e.secIDs(i)).secStart;
                        e.fixon(ctr) = eyeData(e.trialno).section(e.secIDs(i)).secStart - ...
                            e.trialOnset;
                        ctr=ctr+1;
                    end
                end

                e.fixdur(ctr) = str2double(textParse(4)) - ...
                    eyeData(e.trialno).section(e.fixSectionEnd).secStart;
                
                e.fixon(ctr) = eyeData(e.trialno).section(e.fixSectionEnd).secStart - ...
                   e.trialOnset;

                e.trialval(1:ctr) = e.trialno;
                
                

            %if we're not working with a duplicate trial
            elseif e.trialsplit == 1 && e.trialdupe ~= 1 

                e.trialval(1) = e.trialno-1; %assumes split over only one trial
                e.fixdur(1) = eyeData(e.trialval).section(e.fixSectionStart).secEnd...
                                        - e.fixSectionStartTime;
                e.fixon(1) = e.fixSectionStartTime - e.trialOnset;

                trialswitch=0;
                ctr=2;
                for i = 1:e.nSectionOverlap-2
                    if e.fixIDs(i) == 1
                        trialswitch(i) = 1;
                    end

                    if e.trialswitch(i) == 0
                        e.trialval(ctr) = e.trialno-1;
                    else
                        e.trialval(ctr) = e.trialno;
                    end

                    e.fixdur(ctr) = eyeData(e.trialval(ctr)).section(e.secIDs(i)).secEnd - ...
                        eyeData(e.trialval(ctr)).section(e.secIDs(i)).secStart;
                    e.fixon(ctr) = eyeData(e.trialval(ctr)).section(e.secIDs(i)).secStart - ...
                            eyeData(e.trialval(ctr)).section(1).secStart;
                    ctr=ctr+1;
                end

                e.fixdur(ctr) = str2double(textParse(4)) - ...
                    eyeData(e.trialno).section(e.fixSectionEnd).secStart;
                e.fixon(ctr) = eyeData(e.trialno).section(e.fixSectionEnd).secStart - ...
                    e.trialOnset;

                e.trialval(ctr) = e.trialno;
                
            end


            for i = 1:length(e.fixdur)

                %get x and y position
                if e.fixdur(i) >= e.mindur && isfield(eyeData(e.trialval(i)).section(e.secIDs(i)),'x')
                    eyeData(e.trialval(i)).section(e.secIDs(i)).x(end+1) = str2double(textParse(6));
                    eyeData(e.trialval(i)).section(e.secIDs(i)).y(end+1) = str2double(textParse(7));
                    len = length(eyeData(e.trialval(i)).section(e.secIDs(i)).x);
                    
                    if cfg.adjustROI ~= 2
                        eyeData = checkROI(eyeData, e, cfg, str2double(textParse(6)), ...
                                          str2double(textParse(7)), e.trialval(i), e.secIDs(i), len);
                    end
                    
                    eyeData(e.trialval(i)).section(e.secIDs(i)).tstamp(end+1) = ...
                            e.fixon(i);
                    
                    eyeData(e.trialval(i)).section(e.secIDs(i)).dur(end+1) = e.fixdur(i);
                    
                    
                    %keep a copy
                    if isempty(e.fixes)
                        e.fixes = [e.trialval(i) e.secIDs(i) len str2double(textParse(6))...
                                        str2double(textParse(7)) e.fixon(i) e.fixdur(i)];
                                        
                    else
                        e.fixes = [e.fixes; e.trialval(i) e.secIDs(i) len str2double(textParse(6))...
                                        str2double(textParse(7)) e.fixon(i) e.fixdur(i)];
                    end
                    
                    
                elseif e.fixdur(i) >= e.mindur
                    eyeData(e.trialval(i)).section(e.secIDs(i)).x = str2double(textParse(6));
                    eyeData(e.trialval(i)).section(e.secIDs(i)).y = str2double(textParse(7));
                    
                    if cfg.adjustROI ~= 2
                        eyeData = checkROI(eyeData, e, cfg, str2double(textParse(6)), ...
                                          str2double(textParse(7)), e.trialval(i), e.secIDs(i), 1);
                    end
                    
                    eyeData(e.trialval(i)).section(e.secIDs(i)).tstamp = ...
                            e.fixon(i);
                    eyeData(e.trialval(i)).section(e.secIDs(i)).dur = e.fixdur(i);
                    
                    len = 1;
                    %keep a copy
                    if isempty(e.fixes)
                        e.fixes = [e.trialval(i) e.secIDs(i) len str2double(textParse(6))...
                                        str2double(textParse(7)) e.fixon(i) e.fixdur(i)];
                    else
                        e.fixes = [e.fixes; e.trialval(i) e.secIDs(i) len ...
                                   str2double(textParse(6)) str2double(textParse(7))...
                                   e.fixon(i) e.fixdur(i)];
                    end
                    
                end

                eyeData(e.trialval(i)).section(e.secIDs(i)).eyeused = textParse(2);

            end

            e.splitFixation=0; 
            if ~isfield(eyeData(e.trialval(end)).section(e.secIDs(end)), 'x')
                e.fixcount = 1;
            else
                e.fixcount = length(eyeData(e.trialval(end)).section(e.secIDs(end)).x)+1;
            end
            e = rmfield(e,'fixdur'); e = rmfield(e, 'trialval');
        end


    end %if splitfixation
    
    
    end %end function
    

%--------------------
    function [e eyeData] = CheckTrialInfo(e, eyeData, cfg, textParse)
        
        %RT info
        if isfield(cfg.ELidentifiers, 'RTname') || ~isempty(cfg.ELidentifiers.RTname)
            if length(textParse) >= max([cfg.ELidentifiers.RTpos cfg.ELidentifiers.RTposval])

                compare1 = textParse(cfg.ELidentifiers.RTpos);
                compare2 = cfg.ELidentifiers.RTname;

                if strcmp(compare1, compare2)

                    for ns = 1:e.secno
                       eyeData(e.trialno).section(ns).RT = ...
                           str2double(textParse(cfg.ELidentifiers.RTposval));
                    end
                end

            end
        end
            
        %RT info
        if isfield(cfg.ELidentifiers,'respname') || ~isempty(cfg.ELidentifiers.respname)
            if length(textParse) >= cfg.ELidentifiers.resppos

                compare1 = textParse(cfg.ELidentifiers.resppos);
                compare2 = cfg.ELidentifiers.respname;

                if strcmp(compare1, compare2)
                    
                    for ns = 1:e.secno
                        eyeData(e.trialno).section(ns).resp = ...
                            textParse(cfg.ELidentifiers.respposval);
                    end
                end

            end
        end
            
        
        %other info
        if isfield(cfg.ELidentifiers, 'trialInfopos') && ~isempty(cfg.ELidentifiers.trialInfopos)
        for j = 1:length(cfg.ELidentifiers.trialInfopos)
            if length(textParse) >= max([cfg.ELidentifiers.trialInfopos(j) ...
                                cfg.ELidentifiers.trialInfoposval(j)])

                compare1 = textParse(cfg.ELidentifiers.trialInfopos(j));
                compare2 = cfg.ELidentifiers.trialInfoname{j};

                if strcmp(compare1, compare2)
                    
                    if ~isfield(e,'infoBefore')
                        if e.trialno == 0
                            e.infoBefore = ones(1,1:length(cfg.ELidentifiers.trialInfoposval));
                        else
                            e.infoBefore= zeros(1,1:length(cfg.ELidentifiers.trialInfoposval));
                        end
                    end
                   
                        
                    
                    eval(['eyeData(e.trialno+e.infoBefore(j)).section(1).'...
                        cfg.ELidentifiers.trialInfoname{j} ...
                        '= textParse(cfg.ELidentifiers.trialInfoposval(j));']);
                    
                    break;
                    
                end
            end
        end
        end
        

    end
    

%--------------------
    function eyeData = checkROI(eyeData, e, cfg, x, y, trialno, secno, fixno)
    
    
        %only look for ROIs if they have been specified
        if isfield(cfg.ROI, 'name') && ~isempty(cfg.ROI.name)
            
            for r = 1:length(cfg.ROI.x1)
                ROI = [cfg.ROI.x1(r)-cfg.recalSize(1) cfg.ROI.y1(r)-cfg.recalSize(2)...
                       cfg.ROI.x2(r)+cfg.recalSize(1) cfg.ROI.y2(r)+cfg.recalSize(2)];
                if x > ROI(1) && y > ROI(2) && x < ROI(3) && y < ROI(4)
                    eyeData(trialno).section(secno).ROI{fixno} = cfg.ROI.name{r};
                    break
                else
                    eyeData(trialno).section(secno).ROI{fixno} = 'NA';
                end
            end
            
        end
            
            
    
    end

    