function [eyeData fixlong] = recalibrate(fixations, cfg, eyeData)


    
    %% checks for appropriateness of this algorithm
    
    if size(fixations,1) < 5
        fprintf('\nWARNING: Few fixations available for correction;\nresults may not improve\n');
    end
    
    fprintf('\nRecalibrating...\nPlease cite Vadillo, Street, Beesley & Shanks (2015, BRM)\n');
    
    %% get stim coordinates
    
    
    for i = 1:length(cfg.ROI.name)

        %find centre of stim
        Cx = cfg.ROI.x1(i) + (cfg.ROI.x2(i) - cfg.ROI.x1(i))/2;
        Cy = cfg.ROI.y1(i) + (cfg.ROI.y2(i) - cfg.ROI.y1(i))/2;

        
        if ~exist('stimCoords','var')
            stimCoords = [Cx Cy];
        else
            stimCoords = [stimCoords;Cx Cy];
        end
        
        if cfg.splitROIs ==1 
            
            [s b] = splitROI(cfg.ROI.x1(i),cfg.ROI.y1(i),cfg.ROI.x2(i),cfg.ROI.y2(i),...
                cfg.pixperdegW, cfg.pixperdegH, cfg.splitDeg);
            
            if ~exist('stimCoordsSplit','var')
                stimCoordsSplit = s;
                smallROI = b;
            else
                stimCoordsSplit = [stimCoordsSplit;s];
                smallROI = [smallROI;b];
            end
            clear s;
        end

    end
    
    if exist('stimCoordsSplit','var')
        stimCoordsSplit = stimCoordsSplit';
    end
    stimCoords = stimCoords'; %transpose
    
    
    
    
    %% gather fixations, either by section or by groups of fixations
    if cfg.nFixperGroup == -1
        
        nsec = length(cfg.ELidentifiers.secname)+1;
        
        trialCountr=1; secCountr = 1; countr=1;
        
        while 1
            Fidx = and(fixations(:,1)==trialCountr, fixations(:,2)==secCountr);
            if sum(trialCountr > fixations(:,1)) == size(fixations,1) %if we've run through all trials
                break;
            end
            
            F{countr} = fixations(Fidx,4:5)';
            if secCountr == nsec
                trialCountr=trialCountr+1;
                secCountr=1;
            else
                secCountr=secCountr+1;
            end
            countr = countr+1;
            clear Fidx;
        end
        
   
    elseif size(fixations,1) > cfg.nFixperGroup %if there are more than X fixations in this data file
%         nFix = floor(size(fixations,1)/cfg.nFixperGroup); %number of sets of 15
        remFix = rem(size(fixations,1),cfg.nFixperGroup); %and the remainder
        countr=1;
        
        
        for j = 1:cfg.nFixperGroup:size(fixations,1)
            
            
            if j + cfg.nFixperGroup + remFix >= size(fixations,1)
                F{countr} = fixations(j:end,4:5)';
            else
                %grab a group of fixations
                F{countr} = fixations(j:j+cfg.nFixperGroup-1,4:5)';
            end
            countr=countr+1;
        end
    end
    
    
 
    
    %% run recalibration
    
    nNotCalib = 0;
    
    for i = 1:length(F)

        if cfg.plotRecal == 1 && ~isempty(cfg.ROI.name)
            Fa = F{i};
            scatter(Fa(1,:), Fa(2,:), 'r');
            hold on;
            for r = 1:length(cfg.ROI.name)
                rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                    cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
            end

            axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
            title('ORIGINAL fixations [upside down]');
            if cfg.waitPlot == 1
                pause;
            else
                pause(1);
            end
            clf;
        end



        if length(F{i}) >= 5
            
            %this is the function taken from Vadillo, Street, Beesley & Shanks,
            %Behavior Research Methods. Only perform the correction if there
            %are at least 5 fixations to use for correction.
            if exist('stimCoordsSplit','var')
                C{i} = fixationRecalibration(F{i}, stimCoordsSplit);
            else
                C{i} = fixationRecalibration(F{i}, stimCoords);
            end
        
            if cfg.plotRecal == 1 && ~isempty(cfg.ROI.name)
                Ca = C{i};
                scatter(Ca(1,:), Ca(2,:), 'b');
                hold on;
                for r = 1:length(cfg.ROI.name)
                    rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                        cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
                end
                
                if exist('smallROI','var')
                    for r = 1:size(smallROI,1)
                        rectangle('Position', [smallROI(r,1) smallROI(r,2) smallROI(r,3)-smallROI(r,1) ...
                                    smallROI(r,4)-smallROI(r,2)], 'LineStyle', '--');
                    end
                end

                axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
                title('RECALIBRATED fixations [upside down]');
                if cfg.waitPlot == 1
                    pause;
                else
                    pause(1);
                end
                clf;
            end
        else
            C{i} = F{i};
            nNotCalib = nNotCalib +1;
        end
            
           
    end
    
    if nNotCalib > 0
        fprintf('\nDidn''t recalibrate %i of %i sections\nbecause too few fixations\n', nNotCalib, length(C));
    end
    
    %% check for recalibration size, see if abnormal
    
    warned = 0;
    for j = 1:length(C)
        Ca = C{j};
        Fa = F{j};
        %check the average distance correction. If it's large, produce a
        %warning and don't use the corrected value.
        for k = 1:size(Ca,2)
            dist(k) = sqrt((Ca(1,k) - Fa(1,k))^2 + (Ca(2,k) - Fa(2,k))^2);
            %convert dist (pixels) to degrees of error
            dC(k,:) = [dist(k)/cfg.pixperdegW dist(k)/cfg.pixperdegH];
        end

        degChange(j) = mean(dC(:));
        
        if degChange(j) >= 1.5 && warned == 0
            fprintf('\n\nWARNING: large changes made to data (shift of %1.2f). Use with CAUTION.\nMight indicate people often don''t look at your\ndefined ROIs, or else might indicate you had high calibration error\n', ...
                            degChange(j));
            warned = 1;
            
            
            for n = 1:size(eyeData)
                if isfield(eyeData(n), 'rightErrMax') && ~isempty(eyeData(n).rightErrMax)
                    errmax = eyeData(n).rightErrMax;
                    break;
                end
                if ~exist('errmax', 'var')
                    errmax = -1;
                end
            end
            
            fprintf('\nEyelink-recorded max calibration error (deg): %1.2f\n', errmax);
            
            %plot it out
            subplot(1,2,1);
            scatter(Fa(1,:), Fa(2,:), 'r');
            hold on;
            for r = 1:length(cfg.ROI.name)
                rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                    cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
            end
            
            if exist('smallROI','var')
                for r = 1:size(smallROI,1)
                    rectangle('Position', [smallROI(r,1) smallROI(r,2) smallROI(r,3)-smallROI(r,1) ...
                                smallROI(r,4)-smallROI(r,2)], 'LineStyle', '--');
                end
            end

            axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
            title('ORIGINAL fixations [upside down]');
            
            subplot(1,2,2);
            scatter(Ca(1,:), Ca(2,:), 'b');
            hold on;
            for r = 1:length(cfg.ROI.name)
                rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                    cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
            end
            
            if exist('smallROI','var')
                for r = 1:size(smallROI,1)
                    rectangle('Position', [smallROI(r,1) smallROI(r,2) smallROI(r,3)-smallROI(r,1) ...
                                smallROI(r,4)-smallROI(r,2)], 'LineStyle', '--');
                end
            end

            axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
            title(sprintf('RECALIBRATED fixations [upside down], change = %1.2f', degChange(j)));
            
            useC = input('Use this change? 1=yes, 0=no: ');
            if useC == 0
                Ca = Fa; %replace with the original fixations
            end
            
            clf;
        
        elseif degChange(j) >= 1.5 %if already warned
            
            subplot(1,2,1);
            scatter(Fa(1,:), Fa(2,:), 'r');
            hold on;
            for r = 1:length(cfg.ROI.name)
                rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                    cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
            end

            axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
            title('ORIGINAL fixations [upside down]');
            
            subplot(1,2,2);
            scatter(Ca(1,:), Ca(2,:), 'b');
            hold on;
            for r = 1:length(cfg.ROI.name)
                rectangle('Position', [cfg.ROI.x1(r) cfg.ROI.y1(r) ...
                    cfg.ROI.x2(r)-cfg.ROI.x1(r) cfg.ROI.y2(r)-cfg.ROI.y1(r)]);
            end

            if exist('smallROI','var')
                for r = 1:size(smallROI,1)
                    rectangle('Position', [smallROI(r,1) smallROI(r,2) smallROI(r,3)-smallROI(r,1) ...
                                smallROI(r,4)-smallROI(r,2)], 'LineStyle', '--');
                end
            end
                
            axis([0 cfg.screenwidthpix 0 cfg.screenheightpix]);
            title(sprintf('RECALIBRATED fixations [upside down], change = %1.2f', degChange(j)));
            
            useC = input('Use this change? 1=yes, 0=no: ');
            if useC == 0
                Ca = Fa; %replace with the original fixations
            end
            
            
            clf;
            
        end
    end

        
    
    %% reassign from C back into eyeData in the appropriate places
    
    
    x=sprintf('\nCorrected the data by\nbetween %1.2f to %1.2f degrees.\nUse this? [1=yes]: ', min(degChange), max(degChange));
    useChange = input(x);
    
    if useChange == 1
        
        ctr=0;
        %save long form of the data
        for k = 1:length(C)
            dat = C{k};
            %[e.trialno e.secno e.fixcount x y]
            if ~exist('fixlong','var')
                fixlong = [fixations(ctr+1:size(dat,2),1:3) dat']; %save a copy of the long form data
            else
                fixlong = [fixlong; fixations(ctr+1:ctr+size(dat,2),1:3) dat']; %save a copy of the long form data
            end
            ctr = size(fixations,1);
        end
        
        %reassign back into eyeData
        for i = 1:length(fixlong)
            eyeData(fixlong(i,1)).section(fixlong(i,2)).x(fixlong(i,3)) = fixlong(i,4);
            eyeData(fixlong(i,1)).section(fixlong(i,2)).y(fixlong(i,3)) = fixlong(i,5);
        end
                
    else
        fixlong = fixations;
    end
    
    
end